# coding: utf-8
=begin

 * Name: SiSU

 * Description: a framework for document structuring, publishing and search

 * Author: Ralph Amissah

 * Copyright: (C) 1997 - 2009 Ralph Amissah All Rights Reserved.

 * License: GPL 3 or later:

   SiSU, a framework for document structuring, publishing and search

   Copyright (C) Ralph Amissah

   This program is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation, either version 3 of the License, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program. If not, see <http://www.gnu.org/licenses/>.

   If you have Internet connection, the latest version of the GPL should be
   available at these locations:
   <http://www.fsf.org/licensing/licenses/gpl.html>
   <http://www.gnu.org/licenses/gpl.html>

   <http://www.jus.uio.no/sisu/gpl.fsf/toc.html>
   <http://www.jus.uio.no/sisu/gpl.fsf/doc.html>
   <http://www.jus.uio.no/sisu/gpl.fsf/plain.txt>

 * SiSU uses:
   * Standard SiSU markup syntax,
   * Standard SiSU meta-markup syntax, and the
   * Standard SiSU object citation numbering and system

 * Hompages:
   <http://www.jus.uio.no/sisu>
   <http://www.sisudoc.org>

 * Download:
   <http://www.jus.uio.no/sisu/SiSU/download.html>

 * Ralph Amissah
   <ralph@amissah.com>
   <ralph.amissah@gmail.com>

 ** Description: opendocument text generation

=end
module SiSU_ODF
  require "#{SiSU_lib}/particulars"
  include SiSU_Particulars
  require "#{SiSU_lib}/dal"
  require "#{SiSU_lib}/sysenv"
  include SiSU_Env
  include SiSU_Viz
  require "#{SiSU_lib}/odf_format"
  include SiSU_ODF_format
  require "#{SiSU_lib}/shared_txt"
  require "#{SiSU_lib}/shared_structure"
  @@alt_id_count,@@alt_id_count,@@tablehead,@@number_of_cols=0,0,0,0
  class Source
    require 'zlib'
    require 'find'
    require 'fileutils'
    include FileUtils
    def initialize(opt)
      @opt=opt
      @particulars=SiSU_Particulars::Combined_singleton.instance.get_all(opt)
      @@endnotes_para=[]
    end
    def read
      begin
        @env,@md,@dal_array=@particulars.env,@particulars.md,@particulars.dal_array
        @env.odf_structure
        opendoc=@md.fn[:odf]
        path=@env.path.output_tell
        tool=if @opt.cmd =~/[MVv]/; "#{@env.program.odf_viewer} #{path}/#{@md.fnb}/#{opendoc}"
        else ''
        end
        tell=SiSU_Screen::Ansi.new(@opt.cmd,'Opendocument (ODF:ODT)',tool)
        tell.green_hi_blue unless @opt.cmd =~/q/
        tell=SiSU_Screen::Ansi.new(@opt.cmd,@opt.fns,"#{@env.path.output_tell}/#{@md.fnb}/#{opendoc}")
        tell.flow if @opt.cmd =~/[MV]/
        my_make=SiSU_Env::Create_file.new(@opt.cmd,@opt.fns)
        SiSU_ODF::Source::Scroll.new(@particulars).songsheet
        SiSU_Env::Info_skin.new(@md).select
      rescue; SiSU_Errors::Info_error.new($!,$@,@opt.cmd,@opt.fns).error
      ensure
      end
    end
    private
    class Scroll <Source
      require "#{SiSU_lib}/defaults"
      require "#{SiSU_lib}/shared_txt"
      @@img_count=0
      @@odf={ :body=>[],:open=>[],:close=>[],:head=>[],:metadata=>[],:tail=>[],:endnotes=>[] }
      @@docstart=true
      @@fns,@@dp=nil,nil
      def initialize(particulars)
        @env,@md,@dal_array=particulars.env,particulars.md,particulars.dal_array
        @vz=SiSU_Env::Get_init.instance.skin
        @dp=@@dp ||=SiSU_Env::Info_env.new.digest.pattern
        @regx=/^(?:(?:#{Mx[:br_page]}\s*|#{Mx[:br_page_new]}\s*)?#{Mx[:lv_o]}\d:(\S*?)#{Mx[:lv_c]})?\s*(.+?)\s*#{Mx[:id_o]}~(\d+);(?:\w|[0-6]:)\d+;\w\d+#{Mx[:id_c]}#{Mx[:id_o]}#@dp:#@dp#{Mx[:id_c]}$/
        @serial=/\s*#{Mx[:id_o]}~\d+;(?:\w|[0-6]:)\d+;\w\d+#{Mx[:id_c]}#{Mx[:id_o]}#@dp:#@dp#{Mx[:id_c]}\s*/
        @tab="\t"
        @url_brace=SiSU_Viz::Skin.new.url_decoration
        @br=if @md.cmd =~/M/; "\n"
        else ''
        end
      end
      def songsheet
        pre
        @data=markup(@dal_array)
        post
        publish
      end
      # Used for extraction of endnotes from paragraphs
      def extract_endnotes(para='')
        notes=para.scan(/#{Mx[:en_a_o]}(\d+\s+.+?)\s*#{Mx[:id_o]}#@dp#{Mx[:id_c]}#{Mx[:en_a_c]}/)[1]
        @n=[]
        notes.each do |n| #high cost to deal with <br> appropriately within odf, consider
          n=n.dup.to_s
          if n =~/#{Mx[:br_line]}/
            fix=n.split(/#{Mx[:br_line]}/) #watch #added
            fix.each do |x|
              if x =~/\S+/; @n << x
              end
            end
          else              @n << n
          end
        end
      end
      def odf_metadata(meta)
        #meta.el,meta.txt,meta.type,meta.attrib
      end
      def odf_tail
        generator="Generated by: #{@md.sisu_version[:project]} #{@md.sisu_version[:version]} of #{@md.sisu_version[:date_stamp]} (#{@md.sisu_version[:date]})"  if @md.sisu_version[:version]
        lastdone="Last Generated on: #{Time.now}"
        rubyv="Ruby version: #{@md.ruby_version}"
        sc=if @md.sc_info
          "Source file:    #{@md.sc_filename}\nVersion number: #{@md.sc_number}\nVersion date:   #{@md.sc_date}\n"
        else ''
        end
        url=@md.fnb
        fn=@md.fn[:manifest]
        manifest="#{@vz.url_root_http}/#{url}/#{fn}"
        @@odf[:tail] << %{<text:p text:style-name="P1">Available document outputs: <br /> &lt;<text:a xlink:type="simple" xlink:href="#{manifest}">#{manifest}</text:a>&gt;</text:p>}
        @@odf[:tail] << %{\n<text:p text:style-name="P1">SiSU: &lt;<text:a xlink:type="simple" xlink:href="http://www.jus.uio.no/lm">www.jus.uio.no/sisu</text:a>&gt; and &lt;<text:a xlink:type="simple" xlink:href="http://www.sisudoc.org">www.sisudoc.org</text:a>&gt;</text:p>}
        @@odf[:tail] << "\n</office:text></office:body></office:document-content>"
      end
      def heading(para,no)
        para.gsub!(@serial,'')
        para.gsub!(/#{Mx[:mk_o]}:name#\S+?#{Mx[:mk_c]}/,'')
        para.gsub!(/^#{Mx[:lv_o]}(\d):\S*?#{Mx[:lv_c]}\s*/,'')
        m=/#{$1}/
        breakpage=''
        if @md.fns \
        and @md.fns != '' \
        and @md.fns !=@@fns
          @@docstart=true
          @@fns=@md.fns
        end
        unless @@docstart
          breakpage=if (@md.pagenew or @md.pagebreak) \
          and (@md.pagenew =~ m or @md.pagebreak =~m)
            '<text:p text:style-name="P9"> </text:p>'
          else ''
          end
        end
        @@docstart=false
        %{#{breakpage}<text:h text:style-name="Heading_20_#{no}" text:outline-level="#{no}">#{para}</text:h>}
      end
      def image_src(i)
        image_source=if @md.fns =~/\.ss[tm]$/ \
        and FileTest.file?("#{@env.path.image_source_local_tex}/#{i}") #review
          @env.path.image_source_local_tex
        elsif @md.fns =~/\.-ss[tm]$/ \
        and FileTest.file?("#{@env.path.image_source_remote_tex}/#{i}")
          @env.path.image_source_remote_tex
        elsif FileTest.file?("#{@env.path.image_source_tex}/#{i}")
          @env.path.image_source_tex
        else
          tell=SiSU_Screen::Ansi.new(@md.cmd,"ERROR - image:",%{"#{i}" missing},"search locations: #{@env.path.image_source_local_tex},#{@env.path.image_source_remote_tex} and #{@env.path.image_source_tex}")
          tell.error2 unless @md.cmd =~/q/
          nil
        end
      end
      def image_odf(img)
        # copy image to od image directory (unless exists)
        # divide pixel dimension by 37.79485 and retain 3 decimal places
        # x=str.scan(/(aa[^a]+)/)
        m,u=img[1],img[2]
        i=/^(\S+?\.(?:png|jpg|gif))/.match(m).captures.join if m =~/^(\S+?\.(?:png|jpg|gif))/
        c=/^\S+?\.(?:png|jpg|gif)\s+.+?"(.*?)"/.match(m).captures.join if m =~/^\S+?\.(?:png|jpg|gif)\s+.+?"(.*?)"/
        w,h=/(\d+)x(\d+)/.match(m).captures if m =~/\d+x\d+/
        w=(w.to_i/37.79485).to_s
        h=(h.to_i/37.79485).to_s
        h=/([0-9]+\.\d{0,3})/.match(h).captures.join
        w=/([0-9]+\.\d{0,3})/.match(w).captures.join
        image_source=image_src(i)
        pwd=Dir.pwd
        cp("#{image_source}/#{i}","#{@env.path.odf}/Pictures/#{i}") if image_source
        img=if i.to_s =~/jpg|png|gif/ \
        and h.to_s =~/\d/ \
        and w.to_s =~/\d/
          @@img_count +=1
          %{<draw:frame draw:style-name="fr1" draw:name="graphics#{@@img_count}" text:anchor-type="as-char" svg:width="#{w}cm" svg:height="#{h}cm" draw:z-index="2"><draw:image xlink:href="Pictures/#{i}" xlink:type="simple" xlink:show="embed" xlink:actuate="onLoad"/></draw:frame>#{c}} #anchor-type: as-char or paragraph or char or ...
        else %{<text:p text:style-name="P1">[image omitted]</text:p>}
        end
      end
      def image(para)
        para.gsub!(@serial,'')
        m=para.scan(/(#{Mx[:lnk_o]}\s*(.+?)\s*#{Mx[:lnk_c]}((?:https?|file|ftp)\S+|image))/)
        if m; m.each do |i|
            cont,url=i[1],i[2]
            cont.gsub!(/([)(\]\[])/,"\\\\\\1")
            cont.gsub!(/([+?])/,"\\\\\\1") # incorrect handling of +
            url.gsub!(/([+?])/,"\\\\\\1")
            para.sub!(/#{Mx[:lnk_o]}\s*#{cont}\s*#{Mx[:lnk_c]}#{url}/m,image_odf(i)) #watch
            para.gsub!(/\\([)(\]\[?])/,'\1') #clumsy fix
          end
          m=nil
        end
        para
      end
      def text_link_odf(txt,url,trail)
        txt.gsub!(/(\\\+)/,'+') #this is convoluted, and risky :-(
        url.gsub!(/(\\\+)/,'+') #this is convoluted, and risky :-(
        %{<text:a xlink:type="simple" xlink:href="#{url}">#{txt.strip}</text:a>#{trail}}
      end
      def text_link(para)
        para.gsub!(@serial,'')
        m=para.scan(/(#{Mx[:lnk_o]}(.+?)#{Mx[:lnk_c]}((?:https?|file|ftp)\S+?))([;.,]?(?=\s|$))/) #sort
        if m
          m.each do |i|
            txt,url,trail=i[1],i[2]
            txt.gsub!(/([)(\]\[])/,"\\\\\\1")
            txt.gsub!(/([+?*])/,"\\\\\\1") # problems with +
            url.gsub!(/([+?])/,"\\\\\\1") # problems with +
            para.gsub!(/#{Mx[:lnk_o]}\s*#{txt}#{Mx[:lnk_c]}#{url}/m,text_link_odf(txt,url,trail)) #make sure trailing ']' are not caught in url
            para.gsub!(/\\([)(\]\[?])/,'\1') #clumsy fix
          end
          m=nil
        end
        para
      end
      def normal(para)                                                           #P1 - P3
        para.gsub!(@serial,'')
        para.gsub!(/\b[_\\]((?:https?|file|ftp):\/\/\S+?\.[^'"><\s]+?)([;.,]?(?:\s|$))/,
          '<text:a xlink:type="simple" xlink:href="\1">\1</text:a>\2') #http ftp matches escaped, no decoration
        para.gsub!(/((?:^|\s)#{Mx[:lnk_c]})((?:https?|file|ftp):\/\/\S+?\.[^'"><\s]+?)([;.,]?(?=\s|$))/,
          '\1<text:a xlink:type="simple" xlink:href="\2">\2</text:a>\3') #special case \{ e.g. \}http://url
        para.gsub!(/(^|#{Mx[:gl_c]}|\s)((?:https?|file|ftp):\/\/\S+?\.[^'"><\s]+?)([;.,]?(?=\s|$))/,
          %{\\1#{@url_brace.xml_open}<text:a xlink:type="simple" xlink:href="\\2">\\2</text:a>#{@url_brace.xml_close}\\3}) #http ftp matches with decoration
        #para.gsub!(/\b((?:https?|ftp):\/\/\S+?\.[^'"><\s]+?)([;.,]?(?:\s|$))/, also works
          #%{#{@url_brace.xml_open}<text:a xlink:type="simple" xlink:href="\\1">\\1</text:a>#{@url_brace.xml_close}\\2}) #http ftp matches with decoration
        para.gsub!(/([a-zA-Z0-9._-]+@[a-zA-Z0-9._-]+)/,
          %{#{@url_brace.xml_open}<text:a xlink:type="simple" xlink:href="mailto:\\1">\\1</text:a>#{@url_brace.xml_close}}) if para !~/http:\/\// # improve upon, document crash where url contains '@' symbol
        para=case para
        when /^#{Mx[:pa_o]}:i([1-9])#{Mx[:pa_c]}/m
          m=$1
          para.gsub!(/^#{Mx[:pa_o]}:i#{m}#{Mx[:pa_c]}/m,'')
          %{<text:p text:style-name="P1#{m}">#{para}</text:p>}
        else %{<text:p text:style-name="P1">#{para}</text:p>}
        end
        para
      end
      def fontface(para)
        #para=para.gsub(/<b>(.+?)<\/b>/,%{<text:span text:style-name="T1">\\1</text:span>})
        #para=para.gsub(/<i>(.+?)<\/i>/,%{<text:span text:style-name="T2">\\1</text:span>})
      end
      def footnote(para)
        @astx||=10000
        para.gsub!(/#{Mx[:id_o]}#@dp#{Mx[:id_c]}(#{Mx[:en_a_c]}|#{Mx[:en_b_c]})/,'\1')
        #para.gsub!(/<br \/><:i[1-9]>/,'<br />')
        if para =~/#{Mx[:en_a_o]}\d+\s+/
          para=para.gsub(/#{Mx[:en_a_o]}(\d+)\s+(.+?)#{Mx[:en_a_c]}/,'<text:note text:id="ftn\1" text:note-class="footnote"><text:note-citation>\1</text:note-citation><text:note-body><text:p text:style-name="Footnote"> \2</text:p><text:p text:style-name="Footnote"/></text:note-body></text:note>')
        end
        if para=~/#{Mx[:en_a_o]}[*+]+\s/
          asterisk=para.scan(/#{Mx[:en_a_o]}([*+]+)\s+(.+?)#{Mx[:en_a_c]}/)
          asterisk.each do |x|
            a=x[0].gsub(/([*+])/,"\\\\\\1")
            para=para.gsub(/#{Mx[:en_a_o]}(#{a})\s+(.+?)#{Mx[:en_a_c]}/,%{<text:note text:id="ftn#{@astx.to_s}" text:note-class="footnote"><text:note-citation text:label="\\1">\\1</text:note-citation><text:note-body><text:p text:style-name="Footnote"> \\2</text:p><text:p text:style-name="Footnote"/></text:note-body></text:note>})
            @astx+=1
          end
        end
        if para=~/#{Mx[:en_b_o]}[*+]\d+\s/
          asterisk=para.scan(/#{Mx[:en_b_o]}([*+]\d+)\s+(.+?)#{Mx[:en_b_c]}/)
          asterisk.each do |x|
            a=x[0].gsub(/([*+])/,"\\\\\\1")
            para=para.gsub(/#{Mx[:en_b_o]}(#{a})\s+(.+?)#{Mx[:en_b_c]}/,%{<text:note text:id="ftn#{@astx.to_s}" text:note-class="footnote"><text:note-citation text:label="\\1">\\1</text:note-citation><text:note-body><text:p text:style-name="Footnote"> \\2</text:p><text:p text:style-name="Footnote"/></text:note-body></text:note>})
            @astx+=1
          end
        end
        para
      end
      def group_clean(para)
        para.gsub!(/&amp;nbsp;|&nbsp;|#{Mx[:nbsp]}/,'&#160;')
        para.gsub!(/</,'&lt;'); para.gsub!(/>/,'&gt;')
        para.gsub!(/&lt;(text:span text:style-name="T[1-5]"|\/text:span)&gt;/,'<\1>') #works, not ideal
        para.gsub!(/#{Mx[:br_line]}/,'<br />')
        para.gsub!(/&lt;br(?:\s+\/)?&gt;/,'<br />')
        #para.gsub!(/\s\s/,'&#160;&#160;')
        para
      end
      def poem(para)                                                             #P4 #same as group
        para.gsub!(@serial,'')
        para.gsub!(/#{Mx[:gr_o]}verse(?:-end)?#{Mx[:gr_c]}\s*/m,'')
        parray=[]
        para.split(/#{Mx[:br_line]}|#{Mx[:br_nl]}/).each do |parablock|
          parablock=group_clean(parablock)
          parray << %{<text:p text:style-name="P4">#{parablock}</text:p>} if parablock =~/\S+/
        end
        para=parray.join + '<text:p text:style-name="Standard"/>'
      end
      def group(para)                                                            #P4 #same as verse
        para.gsub!(@serial,'')
        para.gsub!(/#{Mx[:gr_o]}group(?:-end)?#{Mx[:gr_c]}\s*/m,'')
        parray=[]
        para.split(/#{Mx[:br_line]}|#{Mx[:br_nl]}/).each do |parablock|
          parablock=group_clean(parablock)
          parablock.gsub!(/&lt;text:a xlink:type="simple" xlink:href="(.+?)"&gt;/m,'<text:a xlink:type="simple" xlink:href="\1">')
          parablock.gsub!(/&lt;(\/text:a)&gt;/,'<\1>')
          parablock.gsub!(/&lt;(text:note text:id=.+?)&gt;/,'<\1>')
          parablock.gsub!(/&lt;(text:p text:style-name="Footnote")&gt;/,'<\1>')
          parablock.gsub!(/&lt;(\/?text:(?:note-citation|note-body|note|p))&gt;/,'<\1>')
          parablock=footnote(parablock)
          parray << %{<text:p text:style-name="P4">#{parablock}</text:p>} if parablock =~/\S+/
        end
        para=parray.join + '<text:p text:style-name="Standard"/>'
      end
      def code(para)                                                             #P5
        para.gsub!(@serial,'')
        para.gsub!(/#{Mx[:gr_o]}code(?:-end)?#{Mx[:gr_c]}\s*/m,'')
        para.gsub!(/\s\s/,'&#160;&#160;')
        parray=[]
        para.split(/#{Mx[:br_line]}|#{Mx[:br_nl]}/).each do |parablock|
          parablock=group_clean(parablock)
          parablock.gsub!(/^\s*$/,'<br />')
          parablock.gsub!(/\b[_\\]((?:https?|file|ftp):\/\/\S+?\.[^'"><\s]+?)([;.,]?(?:\s|$))/,
            '<text:a xlink:type="simple" xlink:href="\1">\1</text:a>\2') #http ftp matches escaped, no decoration
          parray << %{<text:p text:style-name="P5">#{parablock}</text:p>} if parablock =~/\S+/
        end
        para=parray.join + '<text:p text:style-name="Standard"/>'
        para
      end
      def table(para)                                                            #
        if para =~/#{Mx[:gr_o]}Th?.+/ # tables come as single block
          txt_obj={:table =>para}
          table=SiSU_ODF_format::Table.new(@md,txt_obj)
          para=table.table_split
        end
      end
      def odf_structure(md,t_o)
        @md,@t_o=md,t_o
        if t_o.class == Hash
          para =t_o[:txt]            || nil
          lv =t_o[:lv]               || nil
          ocn =t_o[:ocn]             || nil
          hname =t_o[:h_name]        || nil
          #@h_name =t_o[:h_name]      || nil
        #elsif t_o.class == Array
        #  @txt =txt[0]
        #elsif t_o.class == String
        #  @txt =txt
        else
          #@one,@two,@three=one,two,three
          p t_o.class
          p caller
        end
        lv=lv.to_i
        n=lv - 1
        n3=lv + 2
        lv=nil if lv == 0
        para=unless para=~/^#{Mx[:gr_o]}code#{Mx[:gr_c]}/
          para=if para =~/#{Mx[:lnk_o]}\s*\S+?\.(?:png|jpg|gif)\s.+?#{Mx[:lnk_c]}(?:(?:https?|file|ftp):\S+|image)/; image(para)
          elsif para =~/#{Mx[:lnk_o]}.+?#{Mx[:lnk_c]}(?:(?:https?|file|ftp):\S+|image)/; text_link(para)
          else para
          end
        else para
        end
        para=footnote(para)
        if lv
          @@odf[:body] << case lv
          when 1; heading(para,'1') << @br*2
          when 2; heading(para,'2') << @br*2
          when 3; heading(para,'3') << @br*2
          when 4; heading(para,'4') << @br*2
          when 5; heading(para,'5') << @br*2
          when 6; heading(para,'6') << @br*2
          end
        elsif para =~ /#{Mx[:gr_o]}verse(?:-end)?#{Mx[:gr_c]}/
          @@odf[:body] << poem(para)
          @@odf[:body] << @br*2
        elsif para =~ /#{Mx[:gr_o]}group(?:-end)?#{Mx[:gr_c]}/
          @@odf[:body] << group(para)
          @@odf[:body] << @br*2
        elsif para =~ /#{Mx[:gr_o]}code(?:-end)?#{Mx[:gr_c]}/
          @@odf[:body] << code(para)
          @@odf[:body] << @br*2
        elsif para =~ /#{Mx[:gr_o]}Th?/ #elsif para =~ /<!Th?¡/u
          @@odf[:body] << table(para)
          @@odf[:body] << @br*2
        else
          @@odf[:body] << normal(para) # main text, contents, body KEEP
          @@odf[:body] << @br*2
        end
        @@endnotes_para=[]
      end
      # Used to clean words
      def tidywords(wordlist)
        #wordlist.each do |x|
        #  #x.gsub!(/&/,'&amp;') unless x =~/&\S+;/
        #end
        wordlist
      end
      def markup(data)                                                                 # Used for major markup instructions
        safe_characters=/[^a-zA-Z0-9}{\/?,."';:)(><\-_&!@%~#\]\[*=$| \n+`#{Mx[:tc_p]}]/u
        dir=SiSU_Env::Info_env.new(@md.fns)
        @data_mod,@endnotes,@level,@cont,@copen,@odf_contents_close=Array.new(6){[]}
        @rcdc=false
        (0..6).each { |x| @cont[x]=@level[x]=false }
        (4..6).each { |x| @odf_contents_close[x]='' }
        odf_tail #($1,$2)
        fix=[]
        bullet=image_src('bullet_09.png')
        cp("#{bullet}/bullet_09.png","#{@env.path.odf}/Pictures/.") #if image_src('bullet_09.png')
        data.each do |para|
          para.gsub!(/#{Mx[:id_o]}~0;0:0;x\d+#{Mx[:id_c]}/,'') # if book index? remove
          #p para if para =~safe_characters and @md.cmd =~/V/ #KEEP
          #para.gsub!(/&lt;(~\d+;(?:\w|[0-6]:)\d+;\w\d+)&gt;&lt;(#@dp:#@dp)&gt;/,'<\1><\2>')
          para='' if para =~/#{Mx[:lv_o]}\d+:.*?#{Mx[:lv_c]}.+?#{Mx[:pa_non_object_dummy_heading]}/
          para_array=[]
          para.gsub!(/</,'&lt;'); para.gsub!(/>/,'&gt;')
          word=para.scan(/\S+|\n/)
          if word
            word.each do |w| # _ - / # | : ! ^ ~
              unless para =~/^(?:#{Rx[:meta]}|%+ )/m
                w.gsub!(/&#(?:126|152);/,'~') #126 usual
                if w !~/&\S{1,7};/ \
                or w =~/&nbsp;/
                  w.gsub!(/&/,'&amp;') #watch &nbsp;
                end
              end
              para_array << w
            end
            para=para_array.join(' ')
            para=para.strip
          end
          if para =~/#{Mx[:gr_o]}code#{Mx[:gr_c]}/ #code-block: angle brackets special characters
            para.gsub!(/(^|[^}])_</m,'\1&lt;'); para.gsub!(/(^|[^}])_>/m,'\1&gt;')
            para.gsub!(/(^|[^}])_</m,'\1&lt;'); para.gsub!(/(^|[^}])_>/m,'\1&gt;')
          end
          para.gsub!(/#{Mx[:fa_strike_o]}(.+?)#{Mx[:fa_strike_c]}/,'<del>\1</del>')
          para.gsub!(/#{Mx[:fa_insert_o]}(.+?)#{Mx[:fa_insert_c]}/,'<ins>\1</ins>')
          para.gsub!(/#{Mx[:fa_cite_o]}(.+?)#{Mx[:fa_cite_c]}/,'<cite>\1</cite>')
          para.gsub!(/#{Mx[:mk_o]}:name#\S+?#{Mx[:mk_c]}/,'')
          para.gsub!(/#{Mx[:mk_o]}#([a-zA-Z]+)#{Mx[:mk_c]}/,'&\1;')
          para.gsub!(/#{Mx[:mk_o]}(#[0-9]+)#{Mx[:mk_c]}/,'&\1;')
          para.gsub!(/#{Mx[:mk_o]}[~-]##{Mx[:mk_c]}/,'')
          para.gsub!(/^(#{Mx[:pa_o]}:i[1-9]#{Mx[:pa_c]}\s*)?#{Mx[:gl_bullet]}/,'\1<draw:frame draw:style-name="gr1" text:anchor-type="as-char" svg:width="0.22cm" svg:height="0.22cm" draw:z-index="2"><draw:image xlink:href="Pictures/bullet_09.png" xlink:type="simple" xlink:show="embed" xlink:actuate="onLoad"/></draw:frame> ') # bullet_09.png
          #para.gsub!(/^(<:i[1-9]>\s+)?_\*\s+/,'\1<text:span text:style-name="T6">●</text:span> ')     #bullet
          #para.gsub!(/^(<:i[1-9]>\s+)?_\*\s+/,'\1● ') # bullet utf8, make smaller if used
          #para.gsub!(/^_\*\s+/,'<text:span text:style-name="T6">●</text:span> ')     #bullet
          para.gsub!(/^(#{Mx[:pa_o]}:i[1-9]#{Mx[:pa_c]})\s*#{Mx[:gl_bullet]}/,'\1 <text:span text:style-name="T6">·</text:span> ') #bullet
          para.gsub!(/^#{Mx[:gl_bullet]}/,'<text:span text:style-name="T6">·</text:span> ') #bullet
          para.gsub!(/#{Mx[:br_line]}/,'<br />')
          para.gsub!(/#{Mx[:br_page]}|#{Mx[:br_page_new]}/,'<text:p text:style-name="P8"> </text:p>')
          para.gsub!(/&#169;/,'©') #too arbitrary
          para.gsub!(/.+?<-#>/,'')                                           # remove dummy headings (used by html) #check
          para.gsub!(/#{Mx[:fa_bold_o]}(.+?)#{Mx[:fa_bold_c]}/,'<text:span text:style-name="T1">\1</text:span>')
          para.gsub!(/#{Mx[:fa_italics_o]}(.+?)#{Mx[:fa_italics_c]}/,'<text:span text:style-name="T2">\1</text:span>')
          para.gsub!(/#{Mx[:fa_underscore_o]}(.+?)#{Mx[:fa_underscore_c]}/,'<text:span text:style-name="T3">\1</text:span>')
          para.gsub!(/#{Mx[:fa_superscript_o]}(.+?)#{Mx[:fa_superscript_c]}/,'<text:span text:style-name="T4">\1</text:span>')
          para.gsub!(/#{Mx[:fa_subscript_o]}(.+?)#{Mx[:fa_subscript_c]}/,'<text:span text:style-name="T5">\1</text:span>')
          para.gsub!(/­/u,'-')
          para.gsub!(/ /u, ' ')       # space identify
          para.gsub!(/ /u, ' ')       # space identify
          para.gsub!(/·/u,'*')
          para.gsub!(/[­–—]/u,'-')   #— – chk
          para.gsub!(/ < /i,'&#060;')
          para.gsub!(/\\copy(?:right)?\b/,'&#169;')
          para.gsub!(/\\trademark\b|\\tm\b/,'&#174;')
          #para.gsub!(/\43/,'&#35;') ## watch
          #para.gsub!(/$/,'&#36;') #$ watch
          para.gsub!(/\44/,'&#36;') #$ watch
          #para.gsub!(/^·/,'_*') #$ watch
          #para.gsub!(/·/,'*') #$ watch
          para.gsub!(/#{Mx[:br_page]}|#{Mx[:br_page_new]}/,'')                                         # remove page breaks
          para.gsub!(/^\s*#{Mx[:id_o]}~\d+;(?:\w|[0-6]:)\d+;\w\d+#{Mx[:id_c]}#{Mx[:id_o]}#@dp:#@dp#{Mx[:id_c]}/,'') # remove empty lines
          para.gsub!(/<a href=".+?">(.+?)<\/a>/,'\1')
          para.gsub!(/#{Mx[:mk_o]}name#\S+?#{Mx[:mk_c]}/,'')                                       # remove name links
          #para.gsub!(/<a href=".+?">(.+?)<\/a>/im,'\1')
          #para.gsub!(/&nbsp;/,' ')                                            # decide on
          #para.gsub!(/\{(\S+?\.(?:png|jpg)) .+?\}(?:http:\/\/\S+|image)/,"    [ \\1 ]") #"[ #{@env.url.images_local}\/\\1 ]")
          #para.gsub!(/<!TZ.+/,'')
          #para.gsub!(/^<!T.+/,"#@br[table: ] <~#>");
          #p para if para =~safe_characters and @md.cmd =~/M/ #KEEP
          #para.gsub!(/^(\{\S+?\.(?:png|jpg)\s+.+?"(.*?)"\s*\}\S+)/,"\\1 \n [image: \"\\2\"]")
          wordlist=para.scan(/\S+/)
          para=tidywords(wordlist).join(' ').strip
          if para =~/^#{Rx[:meta]}(\S+)\s+(.+?)\Z/m # for headers
            d_meta=SiSU_text_utils::Header_scan.new(@md,para).meta
            if d_meta; odf_metadata(d_meta)
            end
          end
          @rcdc=true if @rcdc==false \
          and (para =~/~metadata/ or para =~/#{Mx[:lv_o]}1:meta#{Mx[:lv_x]}\s*Document Information/)
          if para !~/(^#{Rx[:meta]}|#{Mx[:br_eof]}|#{Mx[:br_endnotes]})/
            if para =~@regx #/.+?<~\d+;\w\d+;\w\d+>.*/ #watch change
              paranum=para[@regx,3]
              @p_num=SiSU_ODF_format::Paragraph_number.new(paranum)
            end
            @sto=SiSU_Structure::Split_text_object.new(@md,para).odt
            #<office:annotation><dc:date>yyyy-mm-ddT00:00:00</dc:date><text:p>#{ocn}</text:p></office:annotation> #followed immediately by paragraph closure
            ### problem in scroll, it appears tables are getting paragraph numbers
            unless @rcdc
              m=/#{Mx[:id_o]}~(\d+);(?:\w|[0-6]:)\d+;\w\d+#{Mx[:id_c]}#{Mx[:id_o]}#@dp:#@dp#{Mx[:id_c]}$/
              if para =~m \
              and para=~/\S+/
                para=case @sto.format
                when /^(1):(\S*)/
                  txt_obj={:txt =>para,:lv =>$1,:ocn =>@sto.ocn,:h_name=>$2}
                  odf_structure(@md,txt_obj)
                  para
                when /^(2):(\S*)/
                  txt_obj={:txt =>para,:lv =>$1,:ocn =>@sto.ocn,:h_name=>$2}
                  odf_structure(@md,txt_obj)
                  para
                when /^(3):(\S*)/
                  txt_obj={:txt =>para,:lv =>$1,:ocn =>@sto.ocn,:h_name=>$2}
                  odf_structure(@md,txt_obj)
                  para
                when /^(4):(\S+)/ # work on see Split_text_object
                  txt_obj={:txt =>para,:lv =>$1,:ocn =>@sto.ocn,:h_name=>$2}
                  odf_structure(@md,txt_obj)
                  # work on see SiSU_text_parts::Split_text_object
                  para
                when /^(5):(\S*)/
                  txt_obj={:txt =>para,:lv =>$1,:ocn =>@sto.ocn,:h_name=>$2}
                  odf_structure(@md,txt_obj)
                  para
                when /^(6):(\S*)/
                  txt_obj={:txt =>para,:lv =>$1,:ocn =>@sto.ocn,:h_name=>$2}
                  odf_structure(@md,txt_obj)
                  para
                else
                  txt_obj={:txt =>para}
                  odf_structure(@md,txt_obj) #watch may be problematic
                  para
                end
              end
              if para =~/<a name="n\d+">/ \
              and para =~/^(-\{{2}~\d+|<!e[:_]\d+!>)/ # -endnote
                para=''
              end
              if (para !~/#{@vz.margin_txt_0}|#{@vz.margin_txt_1}|#{@vz.margin_txt_2}/)
                # i don't get the condition for no paranum
              end
            else
              if para =~ /^(4)~(\S+)/
                txt_obj={:txt =>para,:lv =>$1,:ocn =>@sto.ocn,:h_name=>$2}
                odf_structure(@md,txt_obj)
                para
              elsif para =~/#{Mx[:id_o]}~(\d+);m\d+;[mdv]\d+#{Mx[:id_c]}#{Mx[:id_o]}#@dp:#@dp#{Mx[:id_c]}$/
                txt_obj={:txt =>para}
                odf_structure(@md,txt_obj) #watch may be problematic
                para
              end
            end
            para.gsub!(/<!.+!>/,' ') if para ## Clean Prepared Text
            para.gsub!(/#{Mx[:gr_o]}.+?#{Mx[:gr_c]}/,' ') if para ## CHECK Clean Prepared Text
            para.gsub!(/#{Mx[:tc_o]}.+?#{Mx[:tc_c]}/,' ') if para ## CHECK Clean Prepared Text
            para.gsub!(/<:\S+>/,' ') if para ## Clean Prepared Text
          end
        end
      end
      def pre
        table=if @md.flag_tables
          %{<style:style style:name="Table1" style:family="table"><style:table-properties style:width="16.999cm" table:align="margins"/></style:style>#@br} +
          %{<style:style style:name="Table1.A" style:family="table-column"><style:table-column-properties style:column-width="16.999cm" style:rel-column-width="65535*"/></style:style>#@br} +
          %{<style:style style:name="Table1.B" style:family="table-column"><style:table-column-properties style:column-width="8.499cm" style:rel-column-width="32767*"/></style:style>#@br} +
          %{<style:style style:name="Table1.C" style:family="table-column"><style:table-column-properties style:column-width="5.666cm" style:rel-column-width="21845*"/></style:style>#@br} +
          %{<style:style style:name="Table1.D" style:family="table-column"><style:table-column-properties style:column-width="4.349cm" style:rel-column-width="16383*"/></style:style>#@br} +
          %{<style:style style:name="Table1.E" style:family="table-column"><style:table-column-properties style:column-width="3.399cm" style:rel-column-width="13107*"/></style:style>#@br} +
          %{<style:style style:name="Table1.F" style:family="table-column"><style:table-column-properties style:column-width="2.833cm" style:rel-column-width="10922*"/></style:style>#@br} +
          %{<style:style style:name="Table1.G" style:family="table-column"><style:table-column-properties style:column-width="2.428cm" style:rel-column-width="9362*"/></style:style>#@br} +
          %{<style:style style:name="Table1.H" style:family="table-column"><style:table-column-properties style:column-width="2.124cm" style:rel-column-width="8191*"/></style:style>#@br} +
          %{<style:style style:name="Table2" style:family="table"><style:table-properties style:width="16.999cm" table:align="margins"/></style:style>#@br} +
          %{<style:style style:name="Table2.A" style:family="table-column"><style:table-column-properties style:column-width="16.999cm" style:rel-column-width="65535*"/></style:style>#@br} +
          %{<style:style style:name="Table2.B" style:family="table-column"><style:table-column-properties style:column-width="8.499cm" style:rel-column-width="32767*"/></style:style>#@br} +
          %{<style:style style:name="Table2.C" style:family="table-column"><style:table-column-properties style:column-width="5.666cm" style:rel-column-width="21845*"/></style:style>#@br} +
          %{<style:style style:name="Table2.D" style:family="table-column"><style:table-column-properties style:column-width="4.349cm" style:rel-column-width="16383*"/></style:style>#@br} +
          %{<style:style style:name="Table2.E" style:family="table-column"><style:table-column-properties style:column-width="3.999cm" style:rel-column-width="13107*"/></style:style>#@br} +
          %{<style:style style:name="Table2.F" style:family="table-column"><style:table-column-properties style:column-width="2.833cm" style:rel-column-width="10922*"/></style:style>#@br} +
          %{<style:style style:name="Table2.G" style:family="table-column"><style:table-column-properties style:column-width="2.428cm" style:rel-column-width="9362*"/></style:style>#@br} +
          %{<style:style style:name="Table2.H" style:family="table-column"><style:table-column-properties style:column-width="2.124cm" style:rel-column-width="8191*"/></style:style>#@br} +
          %{<style:style style:name="Table2.I" style:family="table-column"><style:table-column-properties style:column-width="1.8887cm" style:rel-column-width="7281*"/></style:style>#@br} +
          %{<style:style style:name="Table2.J" style:family="table-column"><style:table-column-properties style:column-width="1.6999cm" style:rel-column-width="6553*"/></style:style>#@br} +
          %{<style:style style:name="Table2.K" style:family="table-column"><style:table-column-properties style:column-width="1.5453cm" style:rel-column-width="5957*"/></style:style>#@br} +
          %{<style:style style:name="Table2.L" style:family="table-column"><style:table-column-properties style:column-width="1.416cm" style:rel-column-width="5461*"/></style:style>#@br} +
          %{<style:style style:name="Table2.M" style:family="table-column"><style:table-column-properties style:column-width="1.307" style:rel-column-width="5041*"/></style:style>#@br} +
          %{<style:style style:name="Table2.N" style:family="table-column"><style:table-column-properties style:column-width="1.214cm" style:rel-column-width="4681*"/></style:style>#@br}
        else ''
        end
        breakpage=if @md.pagenew \
        or @md.pagebreak
          ' fo:break-before="page"'
        else ''
        end
        @@odf[:head]<<%{<?xml version="1.0" encoding="UTF-8"?>#@br} +
          %{<office:document-content xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0" xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0" xmlns:table="urn:oasis:names:tc:opendocument:xmlns:table:1.0" xmlns:draw="urn:oasis:names:tc:opendocument:xmlns:drawing:1.0" xmlns:fo="urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:meta="urn:oasis:names:tc:opendocument:xmlns:meta:1.0" xmlns:number="urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0" xmlns:svg="urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0" xmlns:chart="urn:oasis:names:tc:opendocument:xmlns:chart:1.0" xmlns:dr3d="urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0" xmlns:math="http://www.w3.org/1998/Math/MathML" xmlns:form="urn:oasis:names:tc:opendocument:xmlns:form:1.0" xmlns:script="urn:oasis:names:tc:opendocument:xmlns:script:1.0" xmlns:ooo="http://openoffice.org/2004/office" xmlns:ooow="http://openoffice.org/2004/writer" xmlns:oooc="http://openoffice.org/2004/calc" xmlns:dom="http://www.w3.org/2001/xml-events" xmlns:xforms="http://www.w3.org/2002/xforms" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" office:version="1.0"><office:scripts/>#@br} +
          %{<office:font-face-decls><style:font-face style:name="HG Mincho Light J" svg:font-family="&apos;HG Mincho Light J&apos;, &apos;MS Mincho&apos;, &apos;HG Mincho J&apos;, &apos;HG Mincho L&apos;, &apos;HG Mincho&apos;, Mincho, &apos;MS PMincho&apos;, &apos;HG Mincho Light J&apos;, &apos;MS Gothic&apos;, &apos;HG Gothic J&apos;, &apos;HG Gothic B&apos;, &apos;HG Gothic&apos;, Gothic, &apos;MS PGothic&apos;, &apos;Andale Sans UI&apos;, &apos;Arial Unicode MS&apos;, &apos;Lucida Sans Unicode&apos;, Tahoma" style:font-pitch="variable"/><style:font-face style:name="Nimbus Sans L" svg:font-family="&apos;Nimbus Sans L&apos;" style:font-pitch="variable"/><style:font-face style:name="Tahoma" svg:font-family="Tahoma, Lucidasans, &apos;Lucida Sans&apos;, &apos;Arial Unicode MS&apos;" style:font-pitch="variable"/><style:font-face style:name="Nimbus Roman No9 L" svg:font-family="&apos;Nimbus Roman No9 L&apos;" style:font-family-generic="roman" style:font-pitch="variable"/><style:font-face style:name="Bitstream Vera Sans" svg:font-family="&apos;Bitstream Vera Sans&apos;" style:font-family-generic="swiss" style:font-pitch="variable"/></office:font-face-decls>#@br} +
          %{<office:automatic-styles>#@br} +
          %{#{table}#@br} +
          %{<style:style style:name="P1" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:text-align="justify" style:justify-single-word="false"/></style:style>#@br} +
          %{<style:style style:name="P2" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:text-align="justify" style:justify-single-word="false"/></style:style>#@br} + # P1
          %{<style:style style:name="P3" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:text-align="justify" style:justify-single-word="false"/></style:style>#@br} + # P1
          %{<style:style style:name="P4" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties fo:margin-top="0cm" fo:margin-bottom="0cm" fo:line-height="100%" fo:text-align="justify" style:justify-single-word="false"/></style:style>#@br} +
          %{<style:style style:name="P5" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties fo:margin-top="0cm" fo:margin-bottom="0cm" fo:line-height="100%" fo:text-align="start" style:justify-single-word="false"/></style:style>#@br} +
          %{<style:style style:name="P6" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties fo:break-before="page"/></style:style>#@br} +
          %{<style:style style:name="P7" style:family="paragraph" style:parent-style-name="Table_20_Contents"><style:paragraph-properties fo:text-align="justify" style:justify-single-word="false"/></style:style>#@br} +
          %{<style:style style:name="P8" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties fo:break-before="page"/></style:style>#@br} +
          %{<style:style style:name="P9" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties fo:break-after="page"/></style:style>#@br} +
          %{<style:style style:name="P10" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:text-align="justify" style:justify-single-word="false"/></style:style>#@br} + # P1
          %{<style:style style:name="P11" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="1cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +
          %{<style:style style:name="P12" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="2cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +
          %{<style:style style:name="P13" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="3cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +
          %{<style:style style:name="P14" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="4cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +
          %{<style:style style:name="P15" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="5cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +
          %{<style:style style:name="P16" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="6cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +
          %{<style:style style:name="P17" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="7cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +
          %{<style:style style:name="P18" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="8cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +
          %{<style:style style:name="P19" style:family="paragraph" style:parent-style-name="Standard"><style:paragraph-properties  fo:margin-top="0.199cm" fo:margin-bottom="0.199cm" fo:line-height="150%" fo:margin-left="8cm" fo:margin-right="0cm" fo:text-align="justify" style:justify-single-word="false" fo:text-indent="0cm" style:auto-text-indent="false"/></style:style>#@br} +

          %{<style:style style:name="T1" style:family="text"><style:text-properties fo:font-weight="bold" style:font-weight-asian="bold" style:font-weight-complex="bold"/></style:style>#@br} +
          %{<style:style style:name="T2" style:family="text"><style:text-properties fo:font-style="italic" style:font-style-asian="italic" style:font-style-complex="italic"/></style:style>#@br} +
          %{<style:style style:name="T3" style:family="text"><style:text-properties style:text-underline-style="solid" style:text-underline-width="auto" style:text-underline-color="font-color"/></style:style>#@br} +
          %{<style:style style:name="T4" style:family="text"><style:text-properties style:text-position="super 58%"/></style:style>#@br} +
          %{<style:style style:name="T5" style:family="text"><style:text-properties style:text-position="sub 58%"/></style:style>#@br} +
          %{<style:style style:name="T6" style:family="text"><style:text-properties style:font-name="Courier 10 Pitch" fo:font-size="12pt"/></style:style>#@br} +
          %{<style:style style:name="fr1" style:family="graphic" style:parent-style-name="Graphics"><style:graphic-properties style:wrap="none" style:horizontal-pos="left" style:horizontal-rel="paragraph" style:mirror="none" fo:clip="rect(0cm 0cm 0cm 0cm)" draw:luminance="0%" draw:contrast="0%" draw:red="0%" draw:green="0%" draw:blue="0%" draw:gamma="100%" draw:color-inversion="false" draw:image-opacity="100%" draw:color-mode="standard"/></style:style>#@br} +
          %{<style:style style:name="gr1" style:family="graphic"><style:graphic-properties draw:stroke="none" draw:fill="none" draw:textarea-horizontal-align="center" draw:textarea-vertical-align="middle" draw:color-mode="standard" draw:luminance="0%" draw:contrast="0%" draw:gamma="100%" draw:red="0%" draw:green="0%" draw:blue="0%" fo:clip="rect(0cm 0cm 0cm 0cm)" draw:image-opacity="100%" style:mirror="none" style:run-through="background" style:wrap="none" style:vertical-pos="top" style:vertical-rel="baseline" style:horizontal-pos="left" style:horizontal-rel="paragraph" draw:wrap-influence-on-position="once-concurrent" style:flow-with-text="false"/></style:style>#@br} +
          %{<style:style style:name="gr2" style:family="graphic"><style:graphic-properties draw:stroke="none" draw:fill="none" draw:textarea-horizontal-align="center" draw:textarea-vertical-align="middle" draw:color-mode="standard" draw:luminance="0%" draw:contrast="0%" draw:gamma="100%" draw:red="0%" draw:green="0%" draw:blue="0%" fo:clip="rect(0cm 0cm 0cm 0cm)" draw:image-opacity="100%" style:mirror="none" style:run-through="background" style:wrap="none" style:vertical-pos="middle" style:vertical-rel="baseline" style:horizontal-pos="left" style:horizontal-rel="paragraph" draw:wrap-influence-on-position="once-concurrent" style:flow-with-text="false"/></style:style>#@br} +
          %{</office:automatic-styles>#@br} +
          %{<office:body>#@br} +
          %{<office:text><office:forms form:automatic-focus="false" form:apply-design-mode="false"/>#@br} +
          %{<text:sequence-decls><text:sequence-decl text:display-outline-level="0" text:name="Illustration"/><text:sequence-decl text:display-outline-level="0" text:name="Table"/><text:sequence-decl text:display-outline-level="0" text:name="Text"/><text:sequence-decl text:display-outline-level="0" text:name="Drawing"/></text:sequence-decls>}
      end
      def post
      end
      def publish
        divider='='
        content=[]
        data=@data
        content << @@odf[:open]
        content << @@odf[:head]
        content << @@odf[:body]
        content << @@odf[:metadata]
        #content << @@odf[:owner_details] if @md.stmp =~/\w+/ #not used?
        content << @@odf[:tail]
        Output.new(content,@md,@env).odf
        @@odf[:head],@@odf[:body],@@odf[:tail],@@odf[:metadata]=[],[],[],[]
      end
    end
    class Output <Source
      def initialize(content,md,env)
        @content,@md,@env=content,md,env
      end
      def odf                                                           #%odf output
        SiSU_Env::SiSU_file.new(@md).mkdir
        #filename_odf=SiSU_Env::SiSU_file.new(@md,'content.xml').mkfile
        filename="#{@env.path.odf}/content.xml"
        od=File.new(filename,'w+')
        @content.each do |para|                                                # this is a hack
          od.puts para unless para =~/\A\s*\Z/
        end
        od.close
        opendoc=@md.fn[:odf]
        system("
          cd #{@env.path.odf}
          zip -qr #{opendoc} *
          mv #{opendoc} #{@env.path.output}/#{@md.fnb}/.
          cd #{Dir.pwd}
        ")
      end
    end
  end
end
__END__
todo:
* table of contents
* page header/footer?

done:
headings                              *
footnotes                             *
bold underscore italics strikethrough *
superscript subscript                 *
extended ascii set                    *
indents                               *
  [autonomy_markup0.sst]              *
groups
  poem                                *
  code                                *
  tables
images
bullet
line break
page break

notes?? [you could add a note number for every object/paragraph!]

        cd(@env.path.odf)
        structure=[]
        Find.find(@env.path.odf) do |f|
          structure << puts f
        end
        open(opendoc,'wb') do |f|
          zip=Zlib::ZipWriter.new(f)
          structure.each |z| do
            zip << z
          end
          zip.close
        end
        #zip -qr #{opendoc} *
        mv(opendoc,"#{@env.path.output}/#{@md.fnb}/.")
        cd(Dir.pwd)
